@@ -13,7 +13,7 @@ from io import BytesIO
 
 from unittest import skipIf
 
-from lxml.etree import LxmlSyntaxError
+from lxml.etree import CDATA, LxmlSyntaxError
 
 from .common_imports import etree, HelperTestCase
 
@@ -33,6 +33,12 @@ class _XmlFileTestCaseBase(HelperTestCase):
                 xf.write('toast')
         self.assertXml('<test>toast</test>')
 
+    def test_element_write_cdata(self):
+        with etree.xmlfile(self._file) as xf:
+            with xf.element('test'):
+                xf.write(CDATA('toast & jam'))
+        self.assertXml('<test><![CDATA[toast & jam]]></test>')
+
     def test_element_write_empty(self):
         with etree.xmlfile(self._file) as xf:
             with xf.element('test'):
@@ -63,6 +69,20 @@ class _XmlFileTestCaseBase(HelperTestCase):
         self.assertXml('<test>con<toast>tent<taste>inside</taste>'
                        'tnet</toast>noc</test>')
 
+    def test_element_nested_with_cdata(self):
+        with etree.xmlfile(self._file) as xf:
+            with xf.element('test'):
+                xf.write(CDATA('con'))
+                with xf.element('toast'):
+                    xf.write(CDATA('tent'))
+                    with xf.element('taste'):
+                        xf.write(CDATA('inside'))
+                    xf.write(CDATA('tnet'))
+                xf.write(CDATA('noc'))
+        self.assertXml(
+            '<test><![CDATA[con]]><toast><![CDATA[tent]]><taste><![CDATA[inside]]></taste>'
+            '<![CDATA[tnet]]></toast><![CDATA[noc]]></test>')
+
     def test_write_Element(self):
         with etree.xmlfile(self._file) as xf:
             xf.write(etree.Element('test'))
@@ -176,11 +196,18 @@ class _XmlFileTestCaseBase(HelperTestCase):
         self.assertXml(
             '<test>Comments: &lt;!-- text --&gt;\nEntities: &amp;amp;</test>')
 
+    def test_cdata_escaping(self):
+        with etree.xmlfile(self._file) as xf:
+            with xf.element('test'):
+                xf.write(CDATA('Ensure ]]> is escaped using separate CDATA nodes'))
+        self.assertXml(
+            '<test><![CDATA[Ensure ]]]]><![CDATA[> is escaped using separate CDATA nodes]]></test>')
+
     def test_encoding(self):
-        with etree.xmlfile(self._file, encoding='utf16') as xf:
+        with etree.xmlfile(self._file, encoding='utf-16') as xf:
             with xf.element('test'):
                 xf.write('toast')
-        self.assertXml('<test>toast</test>', encoding='utf16')
+        self.assertXml('<test>toast</test>', encoding='utf-16')
 
     def test_buffering(self):
         with etree.xmlfile(self._file, buffered=False) as xf:
@@ -252,6 +279,15 @@ class _XmlFileTestCaseBase(HelperTestCase):
         else:
             self.assertTrue(False)
 
+    def test_failure_preceding_cdata(self):
+        try:
+            with etree.xmlfile(self._file) as xf:
+                xf.write(CDATA('toast & jam'))
+        except etree.LxmlSyntaxError:
+            self.assertTrue(True)
+        else:
+            self.assertTrue(False)
+
     def test_failure_trailing_text(self):
         with etree.xmlfile(self._file) as xf:
             with xf.element('test'):
@@ -263,6 +299,17 @@ class _XmlFileTestCaseBase(HelperTestCase):
             else:
                 self.assertTrue(False)
 
+    def test_failure_trailing_cdata(self):
+        with etree.xmlfile(self._file) as xf:
+            with xf.element('test'):
+                pass
+            try:
+                xf.write(CDATA('toast & jam'))
+            except etree.LxmlSyntaxError:
+                self.assertTrue(True)
+            else:
+                self.assertTrue(False)
+
     def test_failure_trailing_Element(self):
         with etree.xmlfile(self._file) as xf:
             with xf.element('test'):
