@@ -1,18 +1,17 @@
-# -*- coding: utf-8 -*-
-
 """
 IO test cases that apply to both etree and ElementTree
 """
 
-from __future__ import absolute_import
 
+import pathlib
 import unittest
 import tempfile, gzip, os, os.path, gc, shutil
 
 from .common_imports import (
     etree, ElementTree, _str, _bytes,
     SillyFileLike, LargeFileLike, HelperTestCase,
-    read_file, write_to_file, BytesIO, tmpfile
+    read_file, write_to_file, BytesIO, tmpfile,
+    needs_feature,
 )
 
 
@@ -20,7 +19,7 @@ class _IOTestCaseBase(HelperTestCase):
     """(c)ElementTree compatibility for IO functions/methods
     """
     etree = None
-    
+
     def setUp(self):
         """Setting up a minimal tree
         """
@@ -110,7 +109,7 @@ class _IOTestCaseBase(HelperTestCase):
                 after_write = os.listdir(tempfile.gettempdir())
                 self.assertEqual(read_file(filename, 'rb').replace(b'\n', b''),
                                  self.root_str)
-            except (AssertionError, IOError, OSError):
+            except (AssertionError, OSError):
                 print("Before write: %s, after write: %s" % (
                     difference(before_write), difference(after_write))
                 )
@@ -129,7 +128,7 @@ class _IOTestCaseBase(HelperTestCase):
             'invalid_file.xml')
         try:
             self.tree.write(filename)
-        except IOError:
+        except OSError:
             pass
         else:
             self.assertTrue(
@@ -252,9 +251,9 @@ class _IOTestCaseBase(HelperTestCase):
 
     def test_etree_parse_io_error(self):
         # this is a directory name that contains characters beyond latin-1
-        dirnameEN = _str('Directory')
-        dirnameRU = _str('ÐšÐ°Ñ‚Ð°Ð»Ð¾Ð³')
-        filename = _str('nosuchfile.xml')
+        dirnameEN = 'Directory'
+        dirnameRU = 'ÐšÐ°Ñ‚Ð°Ð»Ð¾Ð³'
+        filename = 'nosuchfile.xml'
         dn = tempfile.mkdtemp(prefix=dirnameEN)
         try:
             self.assertRaises(IOError, self.etree.parse, os.path.join(dn, filename))
@@ -262,7 +261,7 @@ class _IOTestCaseBase(HelperTestCase):
             os.rmdir(dn)
         try:
             dn = tempfile.mkdtemp(prefix=dirnameRU)
-        except (IOError, UnicodeEncodeError, UnicodeDecodeError):
+        except (OSError, UnicodeEncodeError, UnicodeDecodeError):
             # Creating the directory might fail on some platforms depending on encodings.
             raise unittest.SkipTest("file system cannot create slavic file names")
         try:
@@ -271,9 +270,9 @@ class _IOTestCaseBase(HelperTestCase):
             os.rmdir(dn)
 
     def test_parse_utf8_bom(self):
-        utext = _str('Søk på nettet')
+        utext = 'Søk på nettet'
         uxml = '<?xml version="1.0" encoding="UTF-8"?><p>%s</p>' % utext
-        bom = _bytes('\\xEF\\xBB\\xBF').decode(
+        bom = b'\\xEF\\xBB\\xBF'.decode(
             "unicode_escape").encode("latin1")
         self.assertEqual(3, len(bom))
         f = tempfile.NamedTemporaryFile(delete=False)
@@ -289,9 +288,9 @@ class _IOTestCaseBase(HelperTestCase):
         self.assertEqual(utext, tree.getroot().text)
 
     def test_iterparse_utf8_bom(self):
-        utext = _str('Søk på nettet')
+        utext = 'Søk på nettet'
         uxml = '<?xml version="1.0" encoding="UTF-8"?><p>%s</p>' % utext
-        bom = _bytes('\\xEF\\xBB\\xBF').decode(
+        bom = b'\\xEF\\xBB\\xBF'.decode(
             "unicode_escape").encode("latin1")
         self.assertEqual(3, len(bom))
         f = tempfile.NamedTemporaryFile(delete=False)
@@ -309,9 +308,9 @@ class _IOTestCaseBase(HelperTestCase):
         self.assertEqual(utext, root.text)
 
     def test_iterparse_utf16_bom(self):
-        utext = _str('Søk på nettet')
+        utext = 'Søk på nettet'
         uxml = '<?xml version="1.0" encoding="UTF-16"?><p>%s</p>' % utext
-        boms = _bytes('\\xFE\\xFF \\xFF\\xFE').decode(
+        boms = b'\\xFE\\xFF \\xFF\\xFE'.decode(
             "unicode_escape").encode("latin1")
         self.assertEqual(5, len(boms))
         xml = uxml.encode("utf-16")
@@ -334,6 +333,45 @@ class _IOTestCaseBase(HelperTestCase):
 class ETreeIOTestCase(_IOTestCaseBase):
     etree = etree
 
+    @needs_feature('zlib')
+    def test_parse_gzip_file_decompress(self):
+        XMLParser = self.etree.XMLParser
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+
+        data = b'<a>' + b'<b/>' * 200 + b'</a>'
+        parser = XMLParser(decompress=True)
+
+        with tempfile.TemporaryDirectory() as temp_dir:
+            gzfile = pathlib.Path(temp_dir) / "input.xml.gz"
+            with gzip.GzipFile(gzfile, mode='wb') as outfile:
+                outfile.write(data)
+
+            root = parse(str(gzfile), parser=parser)
+
+        self.assertEqual(tostring(root), data)
+
+    @needs_feature('zlib')
+    def test_parse_gzip_file_default_no_unzip(self):
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+
+        data = b'<a>' + b'<b/>' * 200 + b'</a>'
+
+        with tempfile.TemporaryDirectory() as temp_dir:
+            gzfile = pathlib.Path(temp_dir) / "input.xml.gz"
+            with gzip.GzipFile(gzfile, mode='wb') as outfile:
+                outfile.write(data)
+
+            try:
+                root = parse(str(gzfile))
+            except self.etree.XMLSyntaxError:
+                pass  # self.assertGreaterEqual(self.etree.LIBXML_VERSION, (2, 15))
+            else:
+                pass  # self.assertLess(self.etree.LIBXML_VERSION, (2, 15))
+                output = tostring(root)
+                self.assertEqual(output, data)
+
     def test_write_compressed_text(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
